package jesse.keeblarcraft.CustomGoals;

import net.minecraft.entity.ai.control.LookControl;
import net.minecraft.entity.ai.goal.Goal;
import net.minecraft.entity.ai.pathing.BirdNavigation;
import net.minecraft.entity.ai.pathing.EntityNavigation;
import net.minecraft.entity.ai.pathing.MobNavigation;
import net.minecraft.entity.passive.AnimalEntity;
import net.minecraft.entity.player.PlayerEntity;
import org.jetbrains.annotations.Nullable;

import java.util.EnumSet;
import java.util.List;
import java.util.Objects;
import java.util.function.Predicate;


public class FollowPlayerCustomGoal extends Goal {
    private final AnimalEntity animal;
    @Nullable
    private final float maxDistance;
    private final Predicate<PlayerEntity> targetPredicate;
    private PlayerEntity target;
    private final EntityNavigation navigation;
    private final float minFollowDistance;
    private final float speed;

    public FollowPlayerCustomGoal(AnimalEntity animal, float maxDistance, float minDistance, float speed) {
        this.animal = animal;
        this.navigation = animal.getNavigation();
        this.maxDistance = maxDistance;
        this.minFollowDistance = minDistance;
        this.speed = speed;

        this.targetPredicate = Objects::nonNull; //make sure the target predicate is not null

        //some mob controls
        this.setControls(EnumSet.of(Goal.Control.MOVE, Goal.Control.LOOK));
        if (!(animal.getNavigation() instanceof MobNavigation) && !(animal.getNavigation() instanceof BirdNavigation)) {
            throw new IllegalArgumentException("Unsupported mob type for FollowMobGoal");
        }
    }

    @Override
    public boolean canStart () {
        List<PlayerEntity> list = this.animal
                .getWorld()
                .getEntitiesByClass(PlayerEntity.class, this.animal.getBoundingBox().expand((double)this.maxDistance), this.targetPredicate);
        if (!list.isEmpty()) {
            for (PlayerEntity player : list) {
                if (!player.isInvisible()) {
                    this.target = player;
                    return true;
                }
            }
        }

        return false;
    }

    @Override
    public boolean shouldContinue () {
        return this.target != null && this.navigation.isIdle() && this.animal.squaredDistanceTo(this.target) > (double) (this.minFollowDistance * this.minFollowDistance);
    }

    @Override
    public void start() {
        this.animal.getNavigation().startMovingTo(target, 0.5);
    }

    @Override
    public void stop() {
        this.target = null;
        this.navigation.stop();
    }

    @Override
    public void tick() {
        if (this.target == null && this.animal.isLeashed()) {
            this.animal.getLookControl().lookAt(this.target, (float) 10, (float) this.animal.getMaxLookPitchChange());
            double x = this.animal.getX() - this.target.getX();
            double y = this.animal.getY() - this.target.getY();
            double z = this.animal.getZ() - this.target.getZ();
            double distance = x * x + y * y + z * z;
            if (distance >= this.minFollowDistance * this.minFollowDistance) {
                this.navigation.startMovingTo(this.target, this.speed);
            } else {
                this.navigation.stop();
                //LookControl lookControl = this.animal.getLookControl();
//                if (distance <= (double)this.minFollowDistance
//                        || lookControl.getLookX() == this.animal.getX() && lookControl.getLookY() == this.animal.getY() && lookControl.getLookZ() == this.animal.getZ()) {
//                    double h = this.target.getX() - this.animal.getX();
//                    double i = this.target.getZ() - this.animal.getZ();
//                    this.navigation.startMovingTo(this.animal.getX() - h, this.animal.getY(), this.animal.getZ() - i, this.speed);
//                }
            }
        }
    }

}
